#!/usr/bin/env python3
#-*- coding: iso-8859-1 -*-
###############################################################################

__all__ = [ "process_request" ]

import interlocked_counter; from interlocked_counter import InterlockedCounter

_key_counter = InterlockedCounter(3)
_value_counter = InterlockedCounter(10)

def process_request(request: dict, response: dict):

    # the exact HTTP request is ignored, just an entry point

    key, value = _key_counter.next(), _value_counter.next()

    # these requests go straight to the database

    rs = pmnc.transaction.db.execute("SELECT v FROM t WHERE k = {k}", k = key)[0]
    if rs:
        pmnc.transaction.db.execute("UPDATE t SET v = {v} WHERE k = {k}", k = key, v = value)
    else:
        pmnc.transaction.db.execute("INSERT INTO t (k, v) VALUES ({k}, {v})", k = key, v = value)

    # now there is a record, we select it through two different resources,
    # in a single transaction, just for fun

    xa = pmnc.transaction.create()
    xa.db.execute("SELECT v FROM t WHERE k = {k}", k = key)         # this goes to the database every time
    xa.db_cached.execute("SELECT v FROM t WHERE k = {k}", k = key)  # this picks up from the cache first
    rs, rs_cached = xa.execute()

    # as no cache key has been specified, the result of the request has been cached
    # under "exactly the same request" key, i.e. key = ("SELECT v FROM t WHERE k = {k}", k = key)

    # it is possible to specify a simpler or more straightforward cache key

    pmnc.transaction.db_cached.execute(
        "SELECT v FROM t WHERE k = {k}", k = key, pool__cache_key = key)

    # or force "write-through" behaviour, for INSERT/UPDATE requests for instance

    pmnc.transaction.db_cached.execute(
        "UPDATE t SET v = {v} WHERE k = {k}", k = key, v = value, pool__cache_key = None)

    # see transaction.py for more advanced usage

    response["headers"]["content-type"] = "text/plain"
    response["content"] = """\
key={k}
value={dv} (direct)
value={cv} (cached)
""".format(k = key, dv = rs[0][0]["v"], cv = rs_cached[0][0]["v"])

###############################################################################
# EOF